@@ -19,8 +19,17 @@ class COINSTACPyNode:
         assert self._mode in COINSTACPyNode._VALID_MODES_, \
             f"Invalid mode : {self._mode}, Use one of: {COINSTACPyNode._VALID_MODES_}"
 
-    def add_phase(self, phase_cls, multi_iterations=False):
-        self._pipeline.add_phase(phase_cls, multi_iterations)
+    def add_phase(self, phase_cls, local_only: bool = False, multi_iterations: bool = False):
+        """
+        :param phase_cls: Custom implementation of ComputationPhase class
+        :type phase_cls: ComputationPhase
+        :param local_only: This phase will run only locally right after the previous phase is done.
+        :type local_only: bool
+        :param multi_iterations: Specifies if it is a multiple iterations phase.
+        :type multi_iterations: bool
+        Note: It is assumed that a normal(default) computation phase will run one round of local-remote.
+        """
+        self._pipeline.add_phase(phase_cls, local_only, multi_iterations)
 
     def _print_logs(self):
         print()
@@ -52,16 +61,21 @@ class COINSTACPyNode:
             if _out:
                 out.update(**_out)
         except Exception as e:
-            raise RuntimeError(f"ERROR! in Phase: *** {phase_key} *** {e}")
+            _tb.print_exc()
+            raise RuntimeError(f"ERROR! in {self._mode}-{data['state']['clientId']} "
+                               f"Phase:{phase_key} *** {e} ***")
 
         output = {"output": out}
 
+        jump_to_next = out.get('jump_to_next', False)
         if self._mode == 'REMOTE':
+            jump_to_next = jump_to_next or _utils.check(all, 'jump_to_next', True, data['input'])
             output['success'] = out.get('success', False)
 
-        self._cache['next_phase'] = self._pipeline.next_phase(
-            out.get('jump_to_next', False)
-        )
+        self._cache['next_phase'] = self._pipeline.next_phase(jump_to_next)
+
+        if self._pipeline.local_only.get(self._cache['next_phase']):
+            output = self.compute(data)
 
         if self._debug:
             self._logs['cache']['POST-COMPUTATION'] = _copy.deepcopy(self._cache)
@@ -78,16 +92,18 @@ class COINSTACPyNode:
         Backward compatibility for the old library. Deprecated now.
         """
         data = _json.loads(_sys.stdin.read())
-        output = {}
         if data.get('cache') and len(data['cache']) > 0:
             self._cache = data['cache']
             self._pipeline.cache = self._cache
+
+        self._debug = False
+        output = self.compute(data)
+        output['cache'] = self._cache
+
         try:
-            self._debug = False
-            output = self.compute(data)
-            output['cache'] = self._cache
             output = _json.dumps(output)
             _sys.stdout.write(output)
         except Exception as e:
             _tb.print_exc()
-            raise Exception(f"{data['state']['clientId']} error {e} \n ### Output: {output} ***")
+            raise Exception(f"Parsing error in {self._mode}-{data['state']['clientId']} *** {e} ***"
+                            f"\n Output: {output}")
